<?php
/*
Plugin Name: Easy Site Options
Description: Allows for easy creation of site options which are, by default, configurable by Editors and Administrators
Version:     0.3.1
Plugin URI:  http://lud.icro.us/wordpress-easy-site-options/
Author:      John Blackbourn
Author URI:  http://johnblackbourn.com/
Text Domain: easy-site-options
Domain Path: /languages/

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.


= @TODO =

 * WPML integration
 * Callback functions on save to sanitize data
 * Radio buttons/select menus

*/

class EasySiteOptions {

	var $preferences = array();
	var $options = array();
	var $defaults = array(
		'section' => 'default',
		'type'    => 'text'
	);

	function __construct() {

		$this->options = get_option( 'site-options', array() );

		add_action( 'init',                       array( $this, 'init' ), 1 );
		add_action( 'load-toplevel_page_options', array( $this, 'admin_init' ) );
		add_action( 'admin_menu',                 array( $this, 'admin_menu' ) );
		add_action( 'admin_bar_menu',             array( $this, 'admin_bar_menu' ), 31 );
		add_action( 'admin_notices',              array( $this, 'admin_notices' ) );

		$this->register_section( 'default', null );

	}

	function admin_menu() {

		if ( empty( $this->preferences ) )
			return;

		$this->admin_screen = add_utility_page(
			__( 'Site Options', 'easy-site-options' ),
			__( 'Options', 'easy-site-options' ),
			'edit_pages',
			'options',
			array( $this, 'options_page' )
		);

	}

	function admin_bar_menu( $wp_admin_bar ) {

		if ( empty( $this->preferences ) )
			return;
		if ( !current_user_can( 'edit_pages' ) )
			return;
		if ( is_admin() )
			return;

		$url = add_query_arg( array(
			'page' => 'options'
		), admin_url( 'admin.php' ) );

		$wp_admin_bar->add_menu( array(
			'parent' => 'site-name',
			'id'     => 'site-options',
			'title'  => __( 'Options', 'easy-site-options' ),
			'href'   => $url
		) );

	}

	function admin_init() {

		if ( empty( $this->preferences ) )
			return;

		# Register settings:

		foreach ( $this->sections as $section_name => $section ) {

			if ( empty( $section->preferences ) )
				continue;

			add_settings_section( $section_name, $section->label, '__return_false', 'site-options' );

			foreach ( $section->preferences as $pref_name ) {
				$pref = $this->preferences[$pref_name];
				$pref->value = get_site_preference( $pref_name );
				add_settings_field( $pref_name, $pref->label, array( $this, "settings_field_{$pref->type}" ), 'site-options', $section_name, (array)$pref );
			}

		}

		# Process POST:

		if ( !isset( $_POST['action'] ) or ( 'update' != $_POST['action'] ) )
			return;
		if ( !isset( $_POST['option_page'] ) or ( 'site-options' != $_POST['option_page'] ) )
			return;

		check_admin_referer( 'site-options-options' );

		update_option( 'site-options', stripslashes_deep( $_POST['site-options'] ) );

		$url = add_query_arg( array(
			'page' => 'options',
			'settings-updated' => 'true'
		), admin_url( 'admin.php' ) );

		wp_redirect( $url );
		die();

	}

	function settings_field_text( $args ) {

		echo '<input type="text" name="site-options[' . esc_attr( $args['name'] ) . ']" class="regular-text" value="' . esc_attr( $args['value'] ) . '" />';

	}

	function settings_field_password( $args ) {

		echo '<input type="password" name="site-options[' . esc_attr( $args['name'] ) . ']" class="regular-text" value="' . esc_attr( $args['value'] ) . '" />';

	}

	function settings_field_email( $args ) {

		echo '<input type="email" name="site-options[' . esc_attr( $args['name'] ) . ']" class="regular-text" value="' . esc_attr( $args['value'] ) . '" />';

	}

	function settings_field_checkbox( $args ) {

		echo '<input type="checkbox" name="site-options[' . esc_attr( $args['name'] ) . ']" value="1"' . checked( $args['value'], true, false ) . '/>';

	}

	function settings_field_textarea( $args ) {

		# http://core.trac.wordpress.org/ticket/20928#comment:1
		echo '<textarea name="site-options[' . esc_attr( $args['name'] ) . ']" class="regular-text" style="width:25em" rows="6">' . esc_textarea( $args['value'] ) . '</textarea>';

	}

	function settings_field_admin_dropdown( $args ) {

		wp_dropdown_users( array(
			'name'     => 'site-options[' . esc_attr( $args['name'] ) . ']',
			'id'       => $args['name'],
			'selected' => $args['value']
		) );

	}

	function admin_notices() {

		if ( empty( $this->preferences ) )
			return;
		if ( $this->admin_screen != get_current_screen()->id )
			return;
		if ( !isset( $_GET['settings-updated'] ) )
			return;

		?>
		<div id="site-options-updated" class="updated">
			<p><strong><?php _e( 'Site options updated.', 'easy-site-options' ); ?></strong></p>
		</div>
		<?php

	}

	function options_page() {

		$url = add_query_arg( array(
			'page' => 'options'
		), admin_url( 'admin.php' ) );

		?>
		<div class="wrap">
			<?php screen_icon( 'options-general' ); ?>
			<h2><?php _e( 'Site Options', 'easy-site-options' ); ?></h2>
			<form action="<?php echo $url; ?>" method="post" id="site-options-form">
				<?php

				settings_fields( 'site-options' );
				do_settings_sections( 'site-options' );
				submit_button();

				?>
			</form>
		</div>
		<?php

	}

	function register_preference( $name, $label, $args = array() ) {

		$args = wp_parse_args( $args, $this->defaults );

		$args['name']  = $name;
		$args['label'] = $label;

		$this->preferences[$name] = (object) $args;

		$this->sections[$args['section']]->preferences[] = $name;

	}

	function register_section( $name, $label ) {

		$preferences = array();
		$this->sections[$name] = (object) compact( 'label', 'preferences' );

	}

	function get_preference( $name ) {

		if ( !isset( $this->preferences[$name] ) )
			return false;

		if ( !isset( $this->options[$name] ) )
			return false;

		$op = $this->options[$name];

		switch ( $this->preferences[$name]->type ) {

			case 'checkbox':
				return !empty( $op );
				break;

			case 'email':
				return sanitize_email( $op );
				break;

			case 'text':
			default:
				return trim( $op );
				break;

		}

	}

	function init() {

		load_plugin_textdomain( 'easy-site-options', false, dirname( plugin_basename( __FILE__ ) ) . '/languages' );

		do_action( 'eso_init' );

	}

}

function register_site_preference( $name, $label, $args = null ) {
	global $easysiteoptions;
	return $easysiteoptions->register_preference( $name, $label, $args );
}

function register_site_preference_section( $name, $label ) {
	global $easysiteoptions;
	return $easysiteoptions->register_section( $name, $label );
}

function get_site_preference( $name ) {
	global $easysiteoptions;
	return $easysiteoptions->get_preference( $name );
}

function site_preference( $name ) {
	echo get_site_preference( $name );
}

global $easysiteoptions;

$easysiteoptions = new EasySiteOptions;

?>